/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.example.order.manager.service;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.List;

import org.bluestemsoftware.open.eoa.example.order.manager.service.fault.OutOfStockFault;
import org.bluestemsoftware.open.eoa.example.order.manager.util.Constants;
import org.bluestemsoftware.open.eoa.example.order.manager.util.DOMUtils;
import org.bluestemsoftware.open.eoa.example.order.manager.vo.AvailabilityResponse;
import org.bluestemsoftware.specification.eoa.application.spring.MyApplication;
import org.bluestemsoftware.specification.eoa.application.spring.MyApplicationContext;
import org.bluestemsoftware.specification.eoa.application.spring.PartnerApplication;
import org.bluestemsoftware.specification.eoa.application.spring.PartnerOperation;
import org.bluestemsoftware.specification.eoa.application.spring.PartnerRole;
import org.bluestemsoftware.specification.eoa.component.intrface.rt.ActionContext;
import org.bluestemsoftware.specification.eoa.component.intrface.rt.FaultContext;
import org.bluestemsoftware.specification.eoa.component.message.rt.Message;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Models 'warehouse manager' application implemented by partner engine which executes at a
 * specific location, e.g. the dallas warehouse, chicago warehouse or atlanta warehouse.
 */
public class WarehouseLocation extends AbstractWarehouse {

    private PartnerApplication partner;

    private WarehouseLocation(PartnerApplication partner) {
        this.partner = partner;
    }

    /**
     * Using customer zip, retrieves an instance of warehouse manager application, i.e. which
     * executes at warehouse location within closest proximity of customer location.
     * 
     * @param zipCode
     * @return
     */
    public static WarehouseLocation getInstance(String zipCode) {

        MyApplication myApplication = MyApplicationContext.getApplication();

        PartnerApplication partner = null;
        int temp = Integer.parseInt(zipCode);
        if (temp > 75200 && temp < 75400) {
            partner = myApplication.getPartner(Constants.DALLAS_WAREHOUSE);
        } else if (temp > 30300 && temp < 31200) {
            partner = myApplication.getPartner(Constants.ATLANTA_WAREHOUSE);
        } else {
            partner = myApplication.getPartner(Constants.CHICAGO_WAREHOUSE);
        }

        return new WarehouseLocation(partner);

    }

    /**
     * Checks item availability at selected warehouse location as modeled by 'partner' instance
     * variable.
     * 
     * 
     * @param partner
     *        warehouse managment system
     * @return String order number if items available to promise
     * @throws OutOfStockException
     *         if items out of stock
     */
    public AvailabilityResponse checkAvailability(String zipCode, List<Element> items) throws OutOfStockFault {

        // retrieve a role performed by partner, i.e which maps to an
        // interface, and the desired operation

        PartnerRole pRole = partner.getPartnerRole(Constants.PARTNER_AVAILABILITY_ROLE);
        PartnerOperation pOperation = pRole.createOperation(Constants.CHECK_AVAILABILITY_OPERATION);

        // populate partner request and check availability of items ordered
        // by customer

        ActionContext pRequest = pOperation.createRequestAction();
        Node zipCodeElement = pRequest.getMessage().evaluateNode(Constants.ZIP_CODE);
        DOMUtils.setText((Element)zipCodeElement, zipCode);
        Element itemsElement = (Element)pRequest.getMessage().evaluateNode(Constants.ITEMS);
        for (Element item : items) {
            itemsElement.appendChild(pRequest.getMessage().importNode(item, true));
        }

        // if items are not available to promise, an out of stock fault
        // is returned containing deficient items

        ActionContext pResponse = pOperation.sendRequest(pRequest, 1000 * 10);
        Message responseMessage = pResponse.getMessage();
        if (pResponse instanceof FaultContext) {
            itemsElement = (Element)responseMessage.evaluateNode(Constants.ITEMS);
            throw new OutOfStockFault(itemsElement);
        } else {
            String orderNumber = responseMessage.evaluateString(Constants.ORDER_NUMBER);
            Element payload = responseMessage.getContent();
            Element shipCostElement = DOMUtils.getFirstChildElement(payload);
            Number shippingCost = null;
            try {
                String text = DOMUtils.getText(shipCostElement);
                shippingCost = NumberFormat.getCurrencyInstance().parse(text);
            } catch (ParseException pe) {
                throw new RuntimeException(pe.getMessage());
            }
            return new AvailabilityResponse(orderNumber, shippingCost.doubleValue());
        }

    }

   /**
     * Schedules order with warehouse management system responsible for fulfilling order as
     * modeled by 'partner' instance variable. Warehouse has already earmarked inventory as
     * 'promised.' Note that response is returned asynchronously via
     * {@link #scheduleOrderResponse(ActionContext)}, i.e. this is a potentially long-running
     * exchange.
     * 
     * @param orderNumber
     */
    public void scheduleOrder(String orderNumber) {
        PartnerRole pRole = partner.getPartnerRole(Constants.PARTNER_ORDER_PROCESSOR_ROLE);
        PartnerOperation pOperation = pRole.createOperation(Constants.SCHEDULE_ORDER_OPERATION);
        ActionContext request = pOperation.createRequestAction();
        Element payload = request.getMessage().getContent();
        payload.setAttribute("orderNumber", orderNumber);
        pOperation.sendRequest(request);
    }

}
